@@ -147,12 +147,16 @@ def openURL(url_base, data=None, method='Get', cookies=None, username=None, pass
     Uses requests library but with additional checks for OGC service exceptions and url formatting.
     Also handles cookies and simple user password authentication.
     """
+    headers = None
+    rkwargs = {}
+
+    rkwargs['timeout'] = timeout
+
     auth = None
     if username and password:
         auth = (username, password)
 
-    headers = {}
-    rkwargs = {'timeout':timeout}
+    rkwargs['auth'] = auth
 
     # FIXUP for WFS in particular, remove xml style namespace
     # @TODO does this belong here?
@@ -161,7 +165,7 @@ def openURL(url_base, data=None, method='Get', cookies=None, username=None, pass
     if method.lower() == 'post':
         try:
             xml = etree.fromstring(data)
-            headers['Content-Type'] = "text/xml"
+            headers = {'Content-Type': 'text/xml'}
         except (ParseError, UnicodeEncodeError):
             pass
 
@@ -169,6 +173,7 @@ def openURL(url_base, data=None, method='Get', cookies=None, username=None, pass
 
     elif method.lower() == 'get':
         rkwargs['params'] = data
+        
     else:
         raise ValueError("Unknown method ('%s'), expected 'get' or 'post'" % method)
 
@@ -177,6 +182,7 @@ def openURL(url_base, data=None, method='Get', cookies=None, username=None, pass
 
     req = requests.request(method.upper(),
                            url_base,
+                           headers=headers,
                            **rkwargs)
 
     if req.status_code in [400, 401]:
@@ -361,7 +367,6 @@ def http_post(url=None, request=None, lang='en-US', timeout=10, username=None, p
     headers = {
         'User-Agent'      : 'OWSLib (https://geopython.github.io/OWSLib)',
         'Content-type'    : 'text/xml',
-        'Content-length'  : '%d' % len(request),
         'Accept'          : 'text/xml',
         'Accept-Language' : lang,
         'Accept-Encoding' : 'gzip,deflate',
@@ -578,3 +583,16 @@ except:  # 2.6
     from ordereddict import OrderedDict
 
 
+def which_etree():
+    """decipher which etree library is being used by OWSLib"""
+
+    which_etree = None
+
+    if 'lxml' in etree.__file__:
+        which_etree = 'lxml.etree'
+    elif 'xml/etree' in etree.__file__:
+        which_etree = 'xml.etree'
+    elif 'elementree' in etree.__file__:
+        which_etree = 'elementtree.ElementTree'
+    
+    return which_etree
