@@ -93,6 +93,10 @@ from owslib.util import (testXMLValue, build_get_url, dump, getTypedValue,
                   getNamespace, element_to_string, nspath, openURL, nspath_eval, log)
 from xml.dom.minidom import parseString
 from owslib.namespaces import Namespaces
+try:                    # Python 3
+    from urllib.parse import urlparse
+except ImportError:     # Python 2
+    from urlparse import urlparse
 
 # namespace definition
 n = Namespaces()
@@ -117,32 +121,42 @@ def get_namespaces():
     return ns
 namespaces = get_namespaces()
 
-class IWebProcessingService():
-    """
-    Abstract interface for an OGC Web Processing Service (WPS).
-    """
-    
-    url = property("""URL for the remote WPS server (string).""")
-    
-    def getcapabilities(**kw):
-        """
-        Makes a GetCapabilities request to the remote WPS server,
-        returns an XML document wrapped in a python file-like object.
-        """
-    
-    def describeprocess(**kw):
-        """
-        Makes a DescribeProcess request to the remote WPS server,
-        returns a Process object containing all the process metadata.
-        """
-        
-    def execute(**kw):
-        """
-        Submits an Execute request to the remote WPS server,
-        returns a WPSExecution object, which can be used to monitor the status of the job, and ultimately retrieve the result.
-        """
 
-class IComplexData():
+def is_reference(val):
+    """
+    Checks if the provided value is a reference (URL).
+    """
+    try:
+        parsed = urlparse(val)
+        is_ref = parsed.scheme != ''
+    except:
+        is_ref = False
+    return is_ref
+
+
+def is_literaldata(val):
+    """
+    Checks if the provided value is a string (includes unicode).
+    """
+    is_str = isinstance(val, str)
+    if not is_str:
+        # on python 2.x we need to check unicode 
+        try:
+            is_str = isinstance(val, unicode)
+        except:
+            # unicode is not available on python 3.x
+            is_str = False
+    return is_str
+
+
+def is_complexdata(val):
+    """
+    Checks if the provided value is an implementation of IComplexData.
+    """
+    return hasattr(val, 'getXml')
+
+
+class IComplexDataInput(object):
     """
     Abstract interface representing complex input object for a WPS request.
     """
@@ -151,7 +165,8 @@ class IComplexData():
         """
         Method that returns the object data as an XML snippet, 
         to be inserted into the WPS request document sent to the server.
-        """ 
+        """
+        raise NotImplementedError
     
 class WebProcessingService(object):
     """
@@ -475,7 +490,7 @@ class WPSExecution():
         identifier: the requested process identifier
         inputs: array of input arguments for the process.
             - LiteralData inputs are expressed as simple (key,value) tuples where key is the input identifier, value is the value
-            - ComplexData inputs are express as (key, object) tuples, where key is the input identifier,
+            - ComplexData inputs are expressed as (key, object) tuples, where key is the input identifier,
               and the object must contain a 'getXml()' method that returns an XML infoset to be included in the WPS request
         output: optional identifier if process output is to be returned as a hyperlink reference
         """
@@ -512,7 +527,8 @@ class WPSExecution():
             #     <wps:LiteralData>dods://igsarm-cida-thredds1.er.usgs.gov:8080/thredds/dodsC/dcp/conus_grid.w_meta.ncml</wps:LiteralData>
             #   </wps:Data>
             # </wps:Input>
-            if isinstance(val, str):
+            if is_literaldata(val):
+                log.debug("literaldata %s", key)
                 dataElement = etree.SubElement(inputElement, nspath_eval('wps:Data', namespaces))
                 literalDataElement = etree.SubElement(dataElement, nspath_eval('wps:LiteralData', namespaces))
                 literalDataElement.text = val
@@ -534,8 +550,11 @@ class WPSExecution():
             #      </wps:Body>
             #   </wps:Reference>
             # </wps:Input>
-            else:
+            elif is_complexdata(val):
+                log.debug("complexdata %s", key)
                 inputElement.append( val.getXml() )
+            else:
+                raise Exception('input type of "%s" parameter is unknown' % key)
         
         # <wps:ResponseForm>
         #   <wps:ResponseDocument storeExecuteResponse="true" status="true">
@@ -728,7 +747,8 @@ class WPSExecution():
         wpsns = getNamespace(root)
 
         self.serviceInstance = root.get( 'serviceInstance' )
-        self.statusLocation = root.get( 'statusLocation' )
+        if self.statusLocation is None:
+            self.statusLocation = root.get( 'statusLocation' )
         
         # <ns0:Status creationTime="2011-11-09T14:19:50Z">
         #  <ns0:ProcessSucceeded>PyWPS Process v.net.path successfully calculated</ns0:ProcessSucceeded>
@@ -791,6 +811,8 @@ class InputOutput(object):
     
     def __init__(self, element):
                 
+        self.abstract = None
+
         # loop over sub-elements without requiring a specific namespace
         for subElement in element:
             
@@ -1127,9 +1149,10 @@ class Process(object):
         self.processVersion = elem.get( nspath('processVersion', ns=wpsns) )
         self.statusSupported = bool( elem.get( "statusSupported" ) )
         self.storeSupported = bool( elem.get( "storeSupported" ) )
+        self.abstract = None
         
         for child in elem:
-            
+
             # this element's namespace
             ns = getNamespace(child)
             
@@ -1162,14 +1185,54 @@ class Process(object):
             if self.verbose==True:
                 dump(self.processOutputs[-1],  prefix='\tOutput: ')
      
-    
-class FeatureCollection():
+
+class ComplexDataInput(IComplexDataInput, ComplexData):
+    def __init__(self, value, mimeType=None, encoding=None, schema=None):
+        super(ComplexDataInput, self).__init__(mimeType=mimeType, encoding=encoding, schema=schema)
+        self.value = value
+        
+    def getXml(self):
+        if is_reference(self.value):
+            return self.complexDataAsReference()
+        else:
+            return self.complexDataRaw()
+
+    def complexDataAsReference(self):
+        """
+           <wps:Reference xlink:href="http://somewhere/test.xml"/>
+        """
+        refElement = etree.Element(nspath_eval('wps:Reference', namespaces),
+                                attrib = { nspath_eval("xlink:href",namespaces) : self.value} )
+        return refElement
+
+    def complexDataRaw(self):
+        '''
+            <wps:Data>
+                <wps:ComplexData mimeType="text/xml" encoding="UTF-8"
+                    schema="http://schemas.opengis.net/gml/3.1.1/base/feature.xsd">
+                </wps:ComplexData>
+            </wps:Data>
+        '''
+        dataElement = etree.Element(nspath_eval('wps:Data', namespaces))
+
+        attrib = dict()
+        if self.encoding:
+            attrib['encoding'] = self.encoding
+        if self.schema:
+            attrib['schema'] = self.schema
+        if self.mimeType:
+            attrib['mimeType'] = self.mimeType
+        complexDataElement = etree.SubElement(dataElement, nspath_eval('wps:ComplexData', namespaces), attrib=attrib )
+        complexDataElement.text = self.value
+        return dataElement
+                
+class FeatureCollection(IComplexDataInput):
     '''
     Base class to represent a Feature Collection used as input to a WPS request.
     The method getXml() is invoked by the WPS execute() method to build the WPS request.
     All subclasses must implement the getXml() method to provide their specific XML.
     
-    Implements IComplexData.
+    Implements IComplexDataInput.
     '''
     
     def __init__(self):
@@ -1221,11 +1284,11 @@ class WFSFeatureCollection(FeatureCollection):
         
         return root
     
-class WFSQuery():
+class WFSQuery(IComplexDataInput):
     '''
     Class representing a WFS query, for insertion into a WFSFeatureCollection instance.
     
-    Implements IComplexData.
+    Implements IComplexDataInput.
     '''
     
     def __init__(self, typeName, propertyNames=[], filters=[]):
